@@ -2,7 +2,6 @@
 extern crate ndarray;
 
 use ndarray::prelude::*;
-use ndarray::Array2;
 
 const INPUT: &'static [u8] = include_bytes!("life.txt");
 //const INPUT: &'static [u8] = include_bytes!("lifelite.txt");
@@ -14,14 +13,14 @@ type Board = Array2<u8>;
 
 fn parse(x: &[u8]) -> Board {
     // make a border of 0 cells
-    let mut map = Board::from_elem(((N + 2) as Ix, (N + 2) as Ix), 0);
+    let mut map = Board::from_elem(((N + 2), (N + 2)), 0);
     let a = Array::from_iter(x.iter().filter_map(|&b| match b {
         b'#' => Some(1),
         b'.' => Some(0),
         _ => None,
     }));
 
-    let a = a.into_shape((N as Ix, N as Ix)).unwrap();
+    let a = a.into_shape((N, N)).unwrap();
     map.slice_mut(s![1..-1, 1..-1]).assign(&a);
     map
 }
@@ -35,7 +34,7 @@ fn parse(x: &[u8]) -> Board {
 fn iterate(z: &mut Board, scratch: &mut Board) {
     // compute number of neighbors
     let mut neigh = scratch.view_mut();
-    neigh.assign_scalar(&0);
+    neigh.fill(0);
     neigh += &z.slice(s![0..-2, 0..-2]);
     neigh += &z.slice(s![0..-2, 1..-1]);
     neigh += &z.slice(s![0..-2, 2..  ]);
@@ -58,7 +57,8 @@ fn iterate(z: &mut Board, scratch: &mut Board) {
 }
 
 fn turn_on_corners(z: &mut Board) {
-    let (n, m) = z.dim();
+    let n = z.rows();
+    let m = z.cols();
     z[[1    , 1    ]] = 1;
     z[[1    , m - 2]] = 1;
     z[[n - 2, 1    ]] = 1;
@@ -66,7 +66,7 @@ fn turn_on_corners(z: &mut Board) {
 }
 
 fn render(a: &Board) {
-    for row in a.inner_iter() {
+    for row in a.genrows() {
         for &x in row {
             if x > 0 {
                 print!("#");
@@ -80,7 +80,7 @@ fn render(a: &Board) {
 
 fn main() {
     let mut a = parse(INPUT);
-    let mut scratch = Board::zeros((N as Ix, N as Ix));
+    let mut scratch = Board::zeros((N, N));
     let steps = 100;
     turn_on_corners(&mut a);
     for _ in 0..steps {
